@@ -1,9 +1,11 @@
 #!/usr/bin/env python
-# Copyright 2010-2013 by Peter Cock.
+# Copyright 2010-2018 by Peter Cock.
 # All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
+#
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 r"""Read and write BGZF compressed files (the GZIP variant used in BAM).
 
 The SAM/BAM file format (Sequence Alignment/Map) comes in a plain text
@@ -16,10 +18,8 @@ file format at http://samtools.sourceforge.net/SAM1.pdf
 Please read the text below about 'virtual offsets' before using BGZF
 files for random access.
 
-
 Aim of this module
 ------------------
-
 The Python gzip library can be used to read BGZF files, since for
 decompression they are just (specialised) gzip files. What this
 module aims to facilitate is random access to BGZF files (using the
@@ -36,13 +36,11 @@ FASTQ, GenBank, etc) or large MAF alignments.
 
 The Bio.SeqIO indexing functions use this module to support BGZF files.
 
-
 Technical Introduction to BGZF
 ------------------------------
-
 The gzip file format allows multiple compressed blocks, each of which
 could be a stand alone gzip file. As an interesting bonus, this means
-you can use Unix "cat" to combined to gzip files into one by
+you can use Unix ``cat`` to combine two or more gzip files into one by
 concatenating them. Also, each block can have one of several compression
 levels (including uncompressed, which actually takes up a little bit
 more space due to the gzip header).
@@ -70,9 +68,9 @@ how big it is from this 'BC' header, and thus seek immediately to
 the second block, and so on.
 
 The BAM indexing scheme records read positions using a 64 bit
-'virtual offset', comprising coffset << 16 | uoffset, where coffset
+'virtual offset', comprising ``coffset << 16 | uoffset``, where ``coffset``
 is the file offset of the BGZF block containing the start of the read
-(unsigned integer using up to 64-16 = 48 bits), and uoffset is the
+(unsigned integer using up to 64-16 = 48 bits), and ``uoffset`` is the
 offset within the (decompressed) block (unsigned 16 bit integer).
 
 This limits you to BAM files where the last block starts by 2^48
@@ -80,13 +78,11 @@ bytes, or 256 petabytes, and the decompressed size of each block
 is at most 2^16 bytes, or 64kb. Note that this matches the BGZF
 'BC' field size which limits the compressed size of each block to
 2^16 bytes, allowing for BAM files to use BGZF with no gzip
-compression (useful for intermediate files in memory to reduced
+compression (useful for intermediate files in memory to reduce
 CPU load).
 
-
 Warning about namespaces
 ------------------------
-
 It is considered a bad idea to use "from XXX import ``*``" in Python, because
 it pollutes the namespace. This is a real issue with Bio.bgzf (and the
 standard Python library gzip) because they contain a function called open
@@ -118,9 +114,8 @@ However, what we recommend instead is to use the explicit namespace, e.g.
 Bio.bgzf
 
 
-Example
--------
-
+Examples
+--------
 This is an ordinary GenBank file compressed using BGZF, so it can
 be decompressed using gzip,
 
@@ -227,14 +222,13 @@ NC_000932.1
 
 from __future__ import print_function
 
-import sys  # to detect when under Python 2
+import sys
 import zlib
 import struct
 
 from Bio._py3k import _as_bytes, _as_string
 from Bio._py3k import open as _open
 
-__docformat__ = "restructuredtext en"
 
 # For Python 2 can just use: _bgzf_magic = '\x1f\x8b\x08\x04'
 # but need to use bytes on Python 3
@@ -299,9 +293,11 @@ def make_virtual_offset(block_start_offset, within_block_offset):
 
     """
     if within_block_offset < 0 or within_block_offset >= 65536:
-        raise ValueError("Require 0 <= within_block_offset < 2**16, got %i" % within_block_offset)
+        raise ValueError("Require 0 <= within_block_offset < 2**16, got %i" %
+                         within_block_offset)
     if block_start_offset < 0 or block_start_offset >= 281474976710656:
-        raise ValueError("Require 0 <= block_start_offset < 2**48, got %i" % block_start_offset)
+        raise ValueError("Require 0 <= block_start_offset < 2**48, got %i" %
+                         block_start_offset)
     return (block_start_offset << 16) | within_block_offset
 
 
@@ -314,7 +310,7 @@ def split_virtual_offset(virtual_offset):
     True
 
     """
-    start = virtual_offset >>16
+    start = virtual_offset >> 16
     return start, virtual_offset ^ (start << 16)
 
 
@@ -395,18 +391,25 @@ def BgzfBlocks(handle):
     data_start = 0
     while True:
         start_offset = handle.tell()
-        # This may raise StopIteration which is perfect here
-        block_length, data = _load_bgzf_block(handle)
+        try:
+            block_length, data = _load_bgzf_block(handle)
+        except StopIteration:
+            break
         data_len = len(data)
         yield start_offset, block_length, data_start, data_len
         data_start += data_len
 
 
 def _load_bgzf_block(handle, text_mode=False):
-    """Internal function to load the next BGZF function (PRIVATE)."""
+    """Load the next BGZF block of compressed data (PRIVATE).
+
+    Returns a tuple (block size and data), or at end of file
+    will raise StopIteration.
+    """
     magic = handle.read(4)
     if not magic:
-        # End of file
+        # End of file - should we signal this differently now?
+        # See https://www.python.org/dev/peps/pep-0479/
         raise StopIteration
     if magic != _bgzf_magic:
         raise ValueError(r"A BGZF (e.g. a BAM file) block should start with "
@@ -435,7 +438,7 @@ def _load_bgzf_block(handle, text_mode=False):
     expected_crc = handle.read(4)
     expected_size = struct.unpack("<I", handle.read(4))[0]
     assert expected_size == len(data), \
-           "Decompressed to %i, not %i" % (len(data), expected_size)
+        "Decompressed to %i, not %i" % (len(data), expected_size)
     # Should cope with a mix of Python platforms...
     crc = zlib.crc32(data)
     if crc < 0:
@@ -443,7 +446,7 @@ def _load_bgzf_block(handle, text_mode=False):
     else:
         crc = struct.pack("<I", crc)
     assert expected_crc == crc, \
-           "CRC is %s, not %s" % (crc, expected_crc)
+        "CRC is %s, not %s" % (crc, expected_crc)
     if text_mode:
         return block_size, _as_string(data)
     else:
@@ -520,6 +523,7 @@ class BgzfReader(object):
     """
 
     def __init__(self, filename=None, mode="r", fileobj=None, max_cache=100):
+        """Initialize the class."""
         # TODO - Assuming we can seek, check for 28 bytes EOF empty block
         # and if missing warn about possible truncation (as in samtools)?
         if max_cache < 1:
@@ -532,8 +536,7 @@ class BgzfReader(object):
             handle = fileobj
             assert "b" in handle.mode.lower()
         else:
-            if "w" in mode.lower() \
-            or "a" in mode.lower():
+            if "w" in mode.lower() or "a" in mode.lower():
                 raise ValueError("Must use read mode (default), not write or append mode")
             handle = _open(filename, "rb")
         self._text = "b" not in mode.lower()
@@ -587,8 +590,9 @@ class BgzfReader(object):
         self._buffers[self._block_start_offset] = self._buffer, block_size
 
     def tell(self):
-        """Returns a 64-bit unsigned BGZF virtual offset."""
-        if 0 < self._within_block_offset == len(self._buffer):
+        """Return a 64-bit unsigned BGZF virtual offset."""
+        if 0 < self._within_block_offset and \
+                self._within_block_offset == len(self._buffer):
             # Special case where we're right at the end of a (non empty) block.
             # For non-maximal blocks could give two possible virtual offsets,
             # but for a maximal block can't use 65536 as the within block
@@ -612,18 +616,20 @@ class BgzfReader(object):
             # (this avoids a function call since _load_block would do nothing)
             self._load_block(start_offset)
             assert start_offset == self._block_start_offset
-        if within_block > len(self._buffer) \
-        and not (within_block == 0 and len(self._buffer)==0):
-            raise ValueError("Within offset %i but block size only %i"
-                             % (within_block, len(self._buffer)))
+        if within_block > len(self._buffer):
+            if not (within_block == 0 and len(self._buffer) == 0):
+                raise ValueError("Within offset %i but block size only %i"
+                                 % (within_block, len(self._buffer)))
         self._within_block_offset = within_block
         # assert virtual_offset == self.tell(), \
         #    "Did seek to %i (%i, %i), but tell says %i (%i, %i)" \
         #    % (virtual_offset, start_offset, within_block,
-        #       self.tell(), self._block_start_offset, self._within_block_offset)
+        #       self.tell(), self._block_start_offset,
+        #       self._within_block_offset)
         return virtual_offset
 
     def read(self, size=-1):
+        """Read method for the BGZF module."""
         if size < 0:
             raise NotImplementedError("Don't be greedy, that could be massive!")
         elif size == 0:
@@ -654,9 +660,10 @@ class BgzfReader(object):
                 return data
 
     def readline(self):
+        """Read a single line for the BGZF file."""
         i = self._buffer.find(self._newline, self._within_block_offset)
         # Three cases to consider,
-        if i==-1:
+        if i == -1:
             # No newline, need to read in more data
             data = self._buffer[self._within_block_offset:]
             self._load_block()  # will reset offsets
@@ -680,6 +687,7 @@ class BgzfReader(object):
             return data
 
     def __next__(self):
+        """Return the next line."""
         line = self.readline()
         if not line:
             raise StopIteration
@@ -691,39 +699,47 @@ class BgzfReader(object):
             return self.__next__()
 
     def __iter__(self):
+        """Iterate over the lines in the BGZF file."""
         return self
 
     def close(self):
+        """Close BGZF file."""
         self._handle.close()
         self._buffer = None
         self._block_start_offset = None
         self._buffers = None
 
     def seekable(self):
+        """Return True indicating the BGZF supports random access."""
         return True
 
     def isatty(self):
+        """Return True if connected to a TTY device."""
         return False
 
     def fileno(self):
+        """Return integer file descriptor."""
         return self._handle.fileno()
 
     def __enter__(self):
+        """Open a file operable with WITH statement."""
         return self
 
     def __exit__(self, type, value, traceback):
+        """Close a file with WITH statement."""
         self.close()
 
 
 class BgzfWriter(object):
+    """Define a BGZFWriter object."""
 
     def __init__(self, filename=None, mode="w", fileobj=None, compresslevel=6):
+        """Initilize the class."""
         if fileobj:
             assert filename is None
             handle = fileobj
         else:
-            if "w" not in mode.lower() \
-            and "a" not in mode.lower():
+            if "w" not in mode.lower() and "a" not in mode.lower():
                 raise ValueError("Must use write or append mode, not %r" % mode)
             if "a" in mode.lower():
                 handle = _open(filename, "ab")
@@ -735,10 +751,12 @@ class BgzfWriter(object):
         self.compresslevel = compresslevel
 
     def _write_block(self, block):
+        """Write provided data to file as a single BGZF compressed block (PRIVATE)."""
         # print("Saving %i bytes" % len(block))
         start_offset = self._handle.tell()
         assert len(block) <= 65536
-        # Giving a negative window bits means no gzip/zlib headers, -15 used in samtools
+        # Giving a negative window bits means no gzip/zlib headers,
+        # -15 used in samtools
         c = zlib.compressobj(self.compresslevel,
                              zlib.DEFLATED,
                              -15,
@@ -746,7 +764,8 @@ class BgzfWriter(object):
                              0)
         compressed = c.compress(block) + c.flush()
         del c
-        assert len(compressed) < 65536, "TODO - Didn't compress enough, try less data in this block"
+        assert len(compressed) < 65536, \
+            "TODO - Didn't compress enough, try less data in this block"
         crc = zlib.crc32(block)
         # Should cope with a mix of Python platforms...
         if crc < 0:
@@ -768,6 +787,7 @@ class BgzfWriter(object):
         self._handle.write(data)
 
     def write(self, data):
+        """Write method for the class."""
         # TODO - Check bytes vs unicode
         data = _as_bytes(data)
         # block_size = 2**16 = 65536
@@ -784,6 +804,7 @@ class BgzfWriter(object):
                 self._buffer = self._buffer[65536:]
 
     def flush(self):
+        """Flush data explicitally."""
         while len(self._buffer) >= 65536:
             self._write_block(self._buffer[:65535])
             self._buffer = self._buffer[65535:]
@@ -792,39 +813,46 @@ class BgzfWriter(object):
         self._handle.flush()
 
     def close(self):
-        """Flush data, write 28 bytes empty BGZF EOF marker, and close the BGZF file."""
+        """Flush data, write 28 bytes BGZF EOF marker, and close BGZF file.
+
+        samtools will look for a magic EOF marker, just a 28 byte empty BGZF
+        block, and if it is missing warns the BAM file may be truncated. In
+        addition to samtools writing this block, so too does bgzip - so this
+        implementation does too.
+        """
         if self._buffer:
             self.flush()
-        # samtools will look for a magic EOF marker, just a 28 byte empty BGZF block,
-        # and if it is missing warns the BAM file may be truncated. In addition to
-        # samtools writing this block, so too does bgzip - so we should too.
         self._handle.write(_bgzf_eof)
         self._handle.flush()
         self._handle.close()
 
     def tell(self):
-        """Returns a BGZF 64-bit virtual offset."""
+        """Return a BGZF 64-bit virtual offset."""
         return make_virtual_offset(self._handle.tell(), len(self._buffer))
 
     def seekable(self):
+        """Return True indicating the BGZF supports random access."""
         # Not seekable, but we do support tell...
         return False
 
     def isatty(self):
+        """Return True if connected to a TTY device."""
         return False
 
     def fileno(self):
+        """Return integer file descriptor."""
         return self._handle.fileno()
 
     def __enter__(self):
+        """Open a file operable with WITH statement."""
         return self
 
     def __exit__(self, type, value, traceback):
+        """Close a file with WITH statement."""
         self.close()
 
 
 if __name__ == "__main__":
-    import sys
     if len(sys.argv) > 1:
         print("Call this with no arguments and pipe uncompressed data in on stdin")
         print("and it will produce BGZF compressed data on stdout. e.g.")
@@ -841,10 +869,19 @@ if __name__ == "__main__":
         print("See also the tool bgzip that comes with samtools")
         sys.exit(0)
 
+    # Ensure we have binary mode handles
+    # (leave stderr as default text mode)
+    if sys.version_info[0] >= 3:
+        stdin = sys.stdin.buffer
+        stdout = sys.stdout.buffer
+    else:
+        stdin = sys.stdin
+        stdout = sys.stdout
+
     sys.stderr.write("Producing BGZF output from stdin...\n")
-    w = BgzfWriter(fileobj=sys.stdout)
+    w = BgzfWriter(fileobj=stdout)
     while True:
-        data = sys.stdin.read(65536)
+        data = stdin.read(65536)
         w.write(data)
         if not data:
             break
