@@ -5,9 +5,6 @@
 
 """Tests for SearchIO BlastIO parsers."""
 
-# For using with statement in Python 2.5 or Jython
-from __future__ import with_statement
-
 import os
 import unittest
 
@@ -24,18 +21,39 @@ def get_file(filename):
     return os.path.join(TEST_DIR, filename)
 
 
-class BlastnTabCases(unittest.TestCase):
+class BlastTabCases(unittest.TestCase):
+
+    def test_tab_2228_tblastn_001(self):
+        "Test parsing TBLASTN 2.2.28+ tabular output (tab_2228_tblastn_001)"
+        tab_file = get_file('tab_2228_tblastn_001.txt')
+        qresults = list(parse(tab_file, FMT,
+                              fields=['evalue', 'sallseqid', 'qseqid'],
+                              comments=True))
+
+        self.assertEqual(1, len(qresults))
+        self.assertEqual(10, len(qresults[0].hits))
+        # there is one hit with an alternative ID
+        self.assertEqual(qresults[0]['gi|148227873|ref|NM_001095167.1|'],
+                qresults[0]['gi|55250552|gb|BC086280.1|'])
+
+        # check some of the HSPs
+        self.assertEqual(0.0, qresults[0][0][0].evalue)
+        self.assertEqual(8e-173, qresults[0][-1][0].evalue)
 
     def test_tab_2228_tblastx_001(self):
         "Test parsing TBLASTX 2.2.28+ tabular output (tab_2228_tblastx_001)"
         tab_file = get_file('tab_2228_tblastx_001.txt')
-        qresults = list(parse(tab_file, FMT, fields=all_fields.values(),
-            comments=True))
+        qresults = list(parse(tab_file, FMT,
+                              fields=list(all_fields.values()),
+                              comments=True))
 
         # this a single query, with 192 hits and 243 hsps
         self.assertEqual(1, len(qresults))
         self.assertEqual(192, len(qresults[0].hits))
         self.assertEqual(243, sum([len(x) for x in qresults[0]]))
+        # there is one hit with an alternative ID
+        self.assertEqual(qresults[0]['gi|31126987|gb|AY255526.2|'],
+                qresults[0]['gi|31342050|ref|NM_181083.2|'])
 
         # only checking the new fields in 2.2.28+
         hit = qresults[0][0]
@@ -67,7 +85,7 @@ class BlastnTabCases(unittest.TestCase):
         counter = 0
 
         # test first qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('gi|16080617|ref|NP_391444.1|', qresult.id)
@@ -112,7 +130,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(31.6, hsp.bitscore)
 
         # test last qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('gi|11464971:4-101', qresult.id)
@@ -152,7 +170,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(32.7, hsp.bitscore)
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
         self.assertEqual(2, counter)
 
     def test_tab_2226_tblastn_002(self):
@@ -162,7 +180,7 @@ class BlastnTabCases(unittest.TestCase):
         qresults = parse(xml_file, FMT)
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
 
     def test_tab_2226_tblastn_003(self):
         "Test parsing TBLASTN 2.2.26+ tabular output (tab_2226_tblastn_003)"
@@ -172,7 +190,7 @@ class BlastnTabCases(unittest.TestCase):
         counter = 0
 
         # test first qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('gi|16080617|ref|NP_391444.1|', qresult.id)
@@ -217,7 +235,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(31.6, hsp.bitscore)
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
         self.assertEqual(1, counter)
 
     def test_tab_2226_tblastn_004(self):
@@ -227,7 +245,7 @@ class BlastnTabCases(unittest.TestCase):
         qresults = parse(xml_file, FMT)
         counter = 0
 
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('gi|11464971:4-101', qresult.id)
@@ -267,7 +285,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(32.7, hsp.bitscore)
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
         self.assertEqual(1, counter)
 
     def test_tab_2226_tblastn_005(self):
@@ -278,7 +296,7 @@ class BlastnTabCases(unittest.TestCase):
         counter = 0
 
         # test first qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -288,7 +306,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(0, len(qresult))
 
         # test second qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -336,7 +354,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(31.6, hsp.bitscore)
 
         # test last qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -379,9 +397,20 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(32.7, hsp.bitscore)
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
         self.assertEqual(3, counter)
 
+    def test_tab_2226_tblastn_005_comments_false(self):
+        "Test parsing TBLASTN 2.2.26+ tabular output with comments (tab_2226_tblastn_005)"
+
+        tab_file = get_file('tab_2226_tblastn_005.txt')
+        exc_msg = ("Encountered unexpected character '#' at the beginning of"
+                   " a line. Set comments=True if the file is a commented"
+                   " file.")
+        qresults = parse(tab_file, FMT)
+        with self.assertRaises(ValueError, msg=exc_msg):
+            next(qresults)
+
     def test_tab_2226_tblastn_006(self):
         "Test parsing TBLASTN 2.2.26+ tabular output with comments (tab_2226_tblastn_006)"
 
@@ -389,7 +418,7 @@ class BlastnTabCases(unittest.TestCase):
         qresults = parse(xml_file, FMT, comments=True)
         counter = 0
 
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -399,7 +428,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(0, len(qresult))
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
         self.assertEqual(1, counter)
 
     def test_tab_2226_tblastn_007(self):
@@ -409,7 +438,7 @@ class BlastnTabCases(unittest.TestCase):
         qresults = parse(xml_file, FMT, comments=True)
         counter = 0
 
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -457,7 +486,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(31.6, hsp.bitscore)
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
         self.assertEqual(1, counter)
 
     def test_tab_2226_tblastn_008(self):
@@ -467,7 +496,7 @@ class BlastnTabCases(unittest.TestCase):
         qresults = parse(xml_file, FMT, comments=True)
         counter = 0
 
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -510,7 +539,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(32.7, hsp.bitscore)
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
         self.assertEqual(1, counter)
 
     def test_tab_2226_tblastn_009(self):
@@ -521,7 +550,7 @@ class BlastnTabCases(unittest.TestCase):
         counter = 0
 
         # test first qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('<unknown program>', qresult.program)
@@ -549,7 +578,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual('gi|16080617|ref|NP_391444.1|', hsp.query_id)
 
         # test last qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('<unknown program>', qresult.program)
@@ -572,7 +601,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual('gi|11464971:4-101', hsp.query_id)
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
         self.assertEqual(2, counter)
 
     def test_tab_2226_tblastn_010(self):
@@ -583,7 +612,7 @@ class BlastnTabCases(unittest.TestCase):
         counter = 0
 
         # test first qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -593,7 +622,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(0, len(qresult))
 
         # test second qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -625,7 +654,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(31.6, hsp.bitscore)
 
         # test last qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -652,7 +681,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(32.7, hsp.bitscore)
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
         self.assertEqual(3, counter)
 
     def test_tab_2226_tblastn_011(self):
@@ -663,7 +692,7 @@ class BlastnTabCases(unittest.TestCase):
         counter = 0
 
         # test first qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -673,7 +702,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(0, len(qresult))
 
         # test second qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -751,7 +780,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(1, hsp.hit_frame)
 
         # test last qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -822,7 +851,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(2, hsp.hit_frame)
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
         self.assertEqual(3, counter)
 
     def test_tab_2226_tblastn_012(self):
@@ -833,7 +862,7 @@ class BlastnTabCases(unittest.TestCase):
         counter = 0
 
         # test first qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -844,7 +873,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(0, len(qresult))
 
         # test second qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -855,7 +884,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(3, len(qresult))
 
         # test last qresult
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('tblastn', qresult.program)
@@ -866,7 +895,7 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual(5, len(qresult))
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
         self.assertEqual(3, counter)
 
     def test_tab_2226_tblastn_013(self):
@@ -876,7 +905,7 @@ class BlastnTabCases(unittest.TestCase):
         qresults = parse(xml_file, FMT, fields="qseq std sseq")
         counter = 0
 
-        qresult = qresults.next()
+        qresult = next(qresults)
         counter += 1
 
         self.assertEqual('<unknown program>', qresult.program)
@@ -927,10 +956,10 @@ class BlastnTabCases(unittest.TestCase):
         self.assertEqual('GLVPDHTLILPVGHYQSMLDLTEEVQTELDQFKSALRKYYLSKGKTCVIYERNFRTQHL', str(hsp.hit.seq))
 
         # check if we've finished iteration over qresults
-        self.assertRaises(StopIteration, qresults.next, )
+        self.assertRaises(StopIteration, next, qresults)
         self.assertEqual(1, counter)
 
 
 if __name__ == "__main__":
-    runner = unittest.TextTestRunner(verbosity = 2)
+    runner = unittest.TextTestRunner(verbosity=2)
     unittest.main(testRunner=runner)
