@@ -7,14 +7,7 @@
 import unittest
 from os import path
 
-from Bio import BiopythonExperimentalWarning
-
-import warnings
-
-
-with warnings.catch_warnings():
-    warnings.simplefilter('ignore', BiopythonExperimentalWarning)
-    from Bio.SearchIO import parse, read
+from Bio.SearchIO import parse, read
 
 
 class HmmpfamTests(unittest.TestCase):
