@@ -11,7 +11,9 @@ to all formats.
 
 """
 
+import pickle
 import unittest
+from io import BytesIO
 from copy import deepcopy
 
 from search_tests_common import compare_search_obj
@@ -61,6 +63,22 @@ class QueryResultCases(unittest.TestCase):
         self.qresult.seq_len = 1102
         self.qresult.target = 'refseq_rna'
 
+    def test_pickle(self):
+        """Test pickling and unpickling of QueryResult"""
+        buf = BytesIO()
+        pickle.dump(self.qresult, buf)
+        unp = pickle.loads(buf.getvalue())
+        self.assertTrue(compare_search_obj(self.qresult, unp))
+
+    def test_order(self):
+        # added hits should be ordered
+        self.assertEqual(self.qresult[0], hit11)
+        self.assertEqual(self.qresult[2], hit31)
+        # removal of second item should bump item #2 to #1
+        del self.qresult['hit2']
+        self.assertEqual(self.qresult[0], hit11)
+        self.assertEqual(self.qresult[1], hit31)
+
     def test_init_none(self):
         """Test QueryResult.__init__, no arguments"""
         qresult = QueryResult()
@@ -88,7 +106,7 @@ class QueryResultCases(unittest.TestCase):
         """Test QueryResult.__iter__"""
         # iteration should return hits contained
         for counter, hit in enumerate(self.qresult):
-            self.assertTrue(hit in (hit11, hit21, hit31))
+            self.assertIn(hit, (hit11, hit21, hit31))
         self.assertEqual(2, counter)
 
     def test_hits(self):
@@ -127,11 +145,19 @@ class QueryResultCases(unittest.TestCase):
     def test_contains(self):
         """Test QueryResult.__contains__"""
         # contains should work with hit ids or hit objects
-        self.assertTrue('hit1' in self.qresult)
-        self.assertTrue(hit21 in self.qresult)
+        self.assertIn('hit1', self.qresult)
+        self.assertIn(hit21, self.qresult)
         self.assertFalse('hit5' in self.qresult)
         self.assertFalse(hit41 in self.qresult)
 
+    def test_contains_alt(self):
+        """Test QueryResult.__contains__, with alternative IDs"""
+        # contains should work with alternative hit IDs
+        hit11._id_alt = ['alt1']
+        query = QueryResult([hit11])
+        self.assertIn('alt1', query)
+        hit11._id_alt = []
+
     def test_len(self):
         """Test QueryResult.__len__"""
         # len() should return the number of hits contained
@@ -153,6 +179,56 @@ class QueryResultCases(unittest.TestCase):
         self.qresult['hit4'] = hit11
         self.assertEqual([hit11, hit21, hit31, hit11], list(self.qresult.hits))
 
+    def test_setitem_ok_alt(self):
+        """Test QueryResult.__setitem__, checking alt hit IDs"""
+        # hit objects assignment should make alt IDs visible
+        hit11._id_alt = ['alt1', 'alt11']
+        query = QueryResult()
+        query['hit1'] = hit11
+        self.assertEqual(hit11, query['hit1'])
+        self.assertEqual(hit11, query['alt1'])
+        self.assertEqual(hit11, query['alt11'])
+        self.assertTrue(hit11.id != 'alt1')
+        self.assertTrue(hit11.id != 'alt11')
+        hit11._id_alt = []
+
+    def test_setitem_ok_alt_existing(self):
+        """Test QueryResult.__setitem__, existing key"""
+        # hit objects assignment on existing hits should also update alt IDs
+        hit11._id_alt = ['alt1']
+        hit21._id_alt = ['alt2']
+        query = QueryResult()
+        query['hit'] = hit11
+        self.assertEqual(hit11, query['hit'])
+        self.assertEqual(hit11, query['alt1'])
+        query['hit'] = hit21
+        self.assertEqual(hit21, query['hit'])
+        self.assertEqual(hit21, query['alt2'])
+        self.assertRaises(KeyError, query.__getitem__, 'alt1')
+        hit11._id_alt = []
+        hit21._id_alt = []
+
+    def test_setitem_ok_alt_ok_promote(self):
+        """Test QueryResult.__setitem__, previously alt ID"""
+        # hit objects assignment with ID previously existing as alternative
+        # should make the ID primary
+        hit11._id_alt = ['alt1']
+        hit41._id_alt = ['alt4']
+        hit31._id_alt = ['alt3']
+        query = QueryResult([hit11, hit41])
+        self.assertEqual(hit11, query['alt1'])
+        self.assertEqual(hit41, query['alt4'])
+        self.assertNotIn('alt1', query._items)
+        self.assertIn('alt1', query._QueryResult__alt_hit_ids)
+        query['alt1'] = hit31
+        self.assertEqual(hit31, query['alt1'])
+        self.assertEqual(hit41, query['alt4'])
+        self.assertIn('alt1', query._items)
+        self.assertNotIn('alt1', query._QueryResult__alt_hit_ids)
+        hit11._id_alt = []
+        hit41._id_alt = []
+        hit31._id_alt = []
+
     def test_setitem_wrong_key_type(self):
         """Test QueryResult.__setitem__, wrong key type"""
         # item assignment should fail if the key is not string
@@ -210,6 +286,33 @@ class QueryResultCases(unittest.TestCase):
         self.assertEqual(1102, new_qresult.seq_len)
         self.assertEqual('refseq_rna', new_qresult.target)
 
+    def test_getitm_slice_alt_ok(self):
+        """Test QueryResult.__getitem__, with slice and alt IDs"""
+        # slicing should be reflected in the alt IDs as well
+        hit31._id_alt = ['alt3']
+        hit11._id_alt = ['alt1']
+        query = QueryResult([hit31, hit11])
+        self.assertEqual(hit11, query['hit1'])
+        self.assertEqual(hit11, query['alt1'])
+        self.assertEqual(hit31, query['hit3'])
+        self.assertEqual(hit31, query['alt3'])
+        query = query[:1]
+        self.assertEqual(hit31, query['hit3'])
+        self.assertEqual(hit31, query['alt3'])
+        self.assertRaises(KeyError, query.__getitem__, 'hit1')
+        self.assertRaises(KeyError, query.__getitem__, 'alt1')
+        hit31._id_alt = []
+        hit11._id_alt = []
+
+    def test_getitem_alt_ok(self):
+        """Test QueryResult.__getitem__, single item with alternative ID"""
+        hit11._id_alt = ['alt1']
+        query = QueryResult([hit11])
+        self.assertEqual(hit11, query['hit1'])
+        self.assertEqual(hit11, query['alt1'])
+        self.assertTrue(hit11.id != 'alt1')
+        hit11._id_alt = []
+
     def test_delitem_string_ok(self):
         """Test QueryResult.__getitem__, with string"""
         # delitem should work with string index
@@ -234,6 +337,18 @@ class QueryResultCases(unittest.TestCase):
         self.assertEqual(1, len(self.qresult))
         self.assertTrue([hit31], self.qresult.hits)
 
+    def test_delitem_alt_ok(self):
+        """Test QueryResult.__delitem__, with alt ID"""
+        # delitem should work with alt IDs
+        hit31._id_alt = ['alt3']
+        qresult = QueryResult([hit31, hit41])
+        self.assertEqual(2, len(qresult))
+        del qresult['alt3']
+        self.assertEqual(1, len(qresult))
+        self.assertEqual(hit41, qresult['hit4'])
+        self.assertRaises(KeyError, qresult.__getitem__, 'alt3')
+        hit31._id_alt = []
+
     def test_description_set(self):
         """Test QueryResult.description setter"""
         # setting the description should change the query seqrecord description
@@ -341,9 +456,27 @@ class QueryResultCases(unittest.TestCase):
 
     def test_append_id_exists(self):
         """Test QueryResult.append, when ID exists"""
-        # append should raise an error if hit_key already exist
+        # append should raise an error if hit_key already exists
         self.assertRaises(ValueError, self.qresult.append, hit11)
 
+    def test_append_alt_id_exists(self):
+        """Test QueryResult.append, when alt ID exists"""
+        # append should raise an error if hit_key already exists as alt ID
+        hit11._id_alt = ['alt']
+        hit21._id_alt = ['alt']
+        qresult = QueryResult([hit11])
+        self.assertRaises(ValueError, qresult.append, hit21)
+        hit11._id_alt = []
+        hit21._id_alt = []
+
+    def test_append_alt_id_exists_alt(self):
+        """Test QueryResult.append, when alt ID exists as primary"""
+        # append should raise an error if alt ID already exists as primary ID
+        hit21._id_alt = ['hit1']
+        qresult = QueryResult([hit11])
+        self.assertRaises(ValueError, qresult.append, hit21)
+        hit21._id_alt = []
+
     def test_hit_filter(self):
         """Test QueryResult.hit_filter"""
         # hit_filter should return a new QueryResult object (shallow copy),
@@ -354,7 +487,7 @@ class QueryResultCases(unittest.TestCase):
         filtered = self.qresult.hit_filter(filter_func)
         self.assertEqual([hit11, hit31], list(filtered.hits))
         # make sure all remaining hits return True for the filter function
-        self.assertTrue(all([filter_func(hit) for hit in filtered]))
+        self.assertTrue(all(filter_func(hit) for hit in filtered))
         self.assertEqual(1102, filtered.seq_len)
         self.assertEqual('refseq_rna', filtered.target)
 
@@ -421,15 +554,15 @@ class QueryResultCases(unittest.TestCase):
         # this would filter out hsp113 and hsp211, effectively removing hit21
         filter_func = lambda hsp: '-' not in str(hsp.fragments[0].query)
         filtered = self.qresult.hsp_filter(filter_func)
-        self.assertTrue('hit1' in filtered)
-        self.assertTrue('hit2' not in filtered)
-        self.assertTrue('hit3' in filtered)
+        self.assertIn('hit1', filtered)
+        self.assertNotIn('hit2', filtered)
+        self.assertIn('hit3', filtered)
         # test hsps in hit11
-        self.assertTrue(all([hsp in filtered['hit1'] for hsp in
-                [hsp111, hsp112, hsp114]]))
+        self.assertTrue(all(hsp in filtered['hit1'] for hsp in
+                            [hsp111, hsp112, hsp114]))
         # test hsps in hit31
-        self.assertTrue(all([hsp in filtered['hit3'] for hsp in
-                [hsp311, hsp312]]))
+        self.assertTrue(all(hsp in filtered['hit3'] for hsp in
+                            [hsp311, hsp312]))
 
     def test_hsp_filter_no_func(self):
         """Test QueryResult.hsp_filter, no arguments"""
@@ -530,12 +663,33 @@ class QueryResultCases(unittest.TestCase):
         self.assertEqual(hit, hit21)
         self.assertEqual([hit11, hit31], list(self.qresult.hits))
 
+    def test_pop_string_alt_ok(self):
+        """Test QueryResult.pop, with alternative ID"""
+        # pop should work with alternative index
+        hit11._id_alt = ['alt1']
+        hit21._id_alt = ['alt2']
+        qresult = QueryResult([hit11, hit21])
+        hit = qresult.pop('alt1')
+        self.assertEqual(hit, hit11)
+        self.assertEqual([hit21], list(qresult))
+        self.assertNotIn('hit1', qresult)
+        hit11._id_alt = []
+        hit21._id_alt = []
+
     def test_index(self):
         """Test QueryResult.index"""
         # index should accept hit objects or hit key strings
         self.assertEqual(2, self.qresult.index('hit3'))
         self.assertEqual(2, self.qresult.index(hit31))
 
+    def test_index_alt(self):
+        """Test QueryResult.index, with alt ID"""
+        # index should work with alt IDs
+        hit11._id_alt = ['alt1']
+        qresult = QueryResult([hit21, hit11])
+        self.assertEqual(1, qresult.index('alt1'))
+        hit11._id_alt = []
+
     def test_index_not_present(self):
         """Test QueryResult.index, when index is not present"""
         self.assertRaises(ValueError, self.qresult.index, 'hit4')
@@ -596,6 +750,13 @@ class HitCases(unittest.TestCase):
         self.hit.evalue = 5e-10
         self.hit.name = 'test'
 
+    def test_pickle(self):
+        """Test pickling and unpickling of Hit"""
+        buf = BytesIO()
+        pickle.dump(self.hit, buf)
+        unp = pickle.loads(buf.getvalue())
+        self.assertTrue(compare_search_obj(self.hit, unp))
+
     def test_init_none(self):
         """Test Hit.__init__, no arguments"""
         hit = Hit()
@@ -617,7 +778,7 @@ class HitCases(unittest.TestCase):
         hit = Hit([hsp111, hsp112, hsp113])
         self.assertEqual('hit1', hit.id)
         self.assertEqual('<unknown description>', hit.description)
-        self.assertEqual('query1', hit.query_id) # set from the HSPs
+        self.assertEqual('query1', hit.query_id)  # set from the HSPs
         self.assertEqual('<unknown description>', hit.query_description)
 
     def test_repr(self):
@@ -642,7 +803,7 @@ class HitCases(unittest.TestCase):
         """Test Hit.__iter__"""
         # iteration should return hsps contained
         for counter, hsp in enumerate(self.hit):
-            self.assertTrue(hsp in [hsp111, hsp112, hsp113])
+            self.assertIn(hsp, [hsp111, hsp112, hsp113])
         self.assertEqual(2, counter)
 
     def test_len(self):
@@ -793,7 +954,7 @@ class HitCases(unittest.TestCase):
         filtered = self.hit.filter(filter_func)
         self.assertEqual([hsp111, hsp113], filtered.hsps)
         # make sure all remaining hits return True for the filter function
-        self.assertTrue(all([filter_func(hit) for hit in filtered]))
+        self.assertTrue(all(filter_func(hit) for hit in filtered))
         self.assertEqual(5e-10, filtered.evalue)
         self.assertEqual('test', filtered.name)
 
@@ -971,6 +1132,13 @@ class HSPMultipleFragmentCases(unittest.TestCase):
         self.frag2.hit_end = 161
         self.hsp = HSP([self.frag1, self.frag2])
 
+    def test_pickle(self):
+        """Test pickling and unpickling of HSP"""
+        buf = BytesIO()
+        pickle.dump(self.hsp, buf)
+        unp = pickle.loads(buf.getvalue())
+        self.assertTrue(compare_search_obj(self.hsp, unp))
+
     def test_len(self):
         """Test HSP.__len__"""
         self.assertEqual(2, len(self.hsp))
@@ -1006,8 +1174,8 @@ class HSPMultipleFragmentCases(unittest.TestCase):
     def test_contains(self):
         """Test HSP.__contains__"""
         frag3 = HSPFragment('hit_id', 'query_id', 'AAA', 'AAT')
-        self.assertTrue(self.frag1 in self.hsp)
-        self.assertTrue(frag3 not in self.hsp)
+        self.assertIn(self.frag1, self.hsp)
+        self.assertNotIn(frag3, self.hsp)
 
     def test_fragments(self):
         """Test HSP.fragments property"""
@@ -1126,7 +1294,7 @@ class HSPFragmentWithoutSeqCases(unittest.TestCase):
         self.assertRaises(TypeError, len, self)
         # len is a shorthand for .aln_span, and it can be set manually
         self.fragment.aln_span = 5
-        self.assertEqual(5,len(self.fragment))
+        self.assertEqual(5, len(self.fragment))
 
     def test_repr(self):
         """Test HSPFragment.__repr__, no alignments"""
@@ -1167,6 +1335,13 @@ class HSPFragmentCases(unittest.TestCase):
         self.fragment = HSPFragment('hit_id', 'query_id', 'ATGCTAGCTACA',
                 'ATG--AGCTAGG')
 
+    def test_pickle(self):
+        """Test pickling and unpickling of HSPFragment"""
+        buf = BytesIO()
+        pickle.dump(self.fragment, buf)
+        unp = pickle.loads(buf.getvalue())
+        self.assertTrue(compare_search_obj(self.fragment, unp))
+
     def test_init_with_seqrecord(self):
         """Test HSPFragment.__init__, with SeqRecord"""
         # init should work with seqrecords
@@ -1394,5 +1569,5 @@ class HSPFragmentCases(unittest.TestCase):
 
 
 if __name__ == "__main__":
-    runner = unittest.TextTestRunner(verbosity = 2)
+    runner = unittest.TextTestRunner(verbosity=2)
     unittest.main(testRunner=runner)
